import pytest
import tinynetrc
import os

HERE = os.path.dirname(os.path.abspath(__file__))


@pytest.fixture()
def netrc():
    return tinynetrc.Netrc(os.path.join(HERE, 'netrc_valid'))


def test_hosts(netrc):
    assert 'mail.google.com' in netrc.hosts
    assert isinstance(netrc.hosts['mail.google.com'], tuple)


def test_machines(netrc):
    assert 'mail.google.com' in netrc.machines
    assert isinstance(netrc.machines['mail.google.com'], dict)
    assert netrc.machines['mail.google.com']['login'] == 'joe@gmail.com'
    assert netrc.machines['mail.google.com']['account'] == 'justagmail'
    assert netrc.machines['mail.google.com']['password'] == 'somethingSecret'


def test_machine_formatting(netrc):
    assert 'login joe@gmail.com' in netrc.format()


def test_machines_can_be_modified(netrc):
    netrc.machines['mail.google.com']['password'] = 'newpassword'
    assert 'newpassword' in netrc.format()


def test_machines_can_be_added(netrc):
    netrc.machines['api.heroku.com']['login'] = 'joe@test.test'
    netrc.machines['api.heroku.com']['password'] = 'supersecret'
    assert 'api.heroku.com' in netrc.format()
    assert 'supersecret' in netrc.format()


def test_machines_can_be_removed(netrc):
    assert 'mail.google.com' in netrc.format()
    del netrc.machines['mail.google.com']
    assert 'mail.google.com' not in netrc.format()



# -*- coding: utf-8 -*-
"""Read and write .netrc files."""
import os
from collections import defaultdict
import netrc

__version__ = '1.0.0'


def dictify_hosts(hosts):
    ret = defaultdict(lambda: {
        'login': None,
        'account': None,
        'password': None,
    })
    for machine, info in hosts.items():
        ret[machine] = {
            'login': info[0],
            'account': info[1],
            'password': info[2],
        }
    return ret


def dedictify_machines(machines):
    return {
        machine: (info.get('login'), info.get('account'), info.get('password'))
        for machine, info
        in machines.items()
    }


class Netrc(object):

    def __init__(self, file=None):
        if file is None:
            try:
                file = os.path.join(os.environ['HOME'], ".netrc")
            except KeyError:
                raise OSError("Could not find .netrc: $HOME is not set")
        self.file = file
        self._netrc = netrc.netrc(file)
        self.machines = dictify_hosts(self._netrc.hosts)

    def authenticators(self, host):
        return self._netrc.authenticators(host)

    @property
    def hosts(self):
        return self._netrc.hosts

    def __repr__(self):
        return repr(dict(self.machines))

    # Adapted from https://github.com/python/cpython/blob/master/Lib/netrc.py
    # to support Python 2
    def format(self):
        """Dump the class data in the format of a .netrc file."""
        self._netrc.hosts = dedictify_machines(self.machines)
        rep = ""
        for host in self._netrc.hosts.keys():
            attrs = self._netrc.hosts[host]
            rep += "machine {host}\n\tlogin {attrs[0]}\n".format(host=host,
                                                                 attrs=attrs)
            if attrs[1]:
                rep += "\taccount {attrs[1]}\n".format(attrs=attrs)
            rep += "\tpassword {attrs[2]}\n".format(attrs=attrs)
        for macro in self._netrc.macros.keys():
            rep += "macdef {macro}\n".format(macro=macro)
            for line in self._netrc.macros[macro]:
                rep += line
            rep += "\n"
        return rep

    def save(self):
        with open(self.file, 'w') as fp:
            fp.write(self.format())



# -*- coding: utf-8 -*-
import re
from setuptools import setup


def find_version(fname):
    """Attempts to find the version number in the file names fname.
    Raises RuntimeError if not found.
    """
    version = ''
    with open(fname, 'r') as fp:
        reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
        for line in fp:
            m = reg.match(line)
            if m:
                version = m.group(1)
                break
    if not version:
        raise RuntimeError('Cannot find version information')
    return version


__version__ = find_version('tinynetrc.py')


def read(fname):
    with open(fname) as fp:
        content = fp.read()
    return content


setup(
    name='tinynetrc',
    version=__version__,
    description='Read and write .netrc files.',
    long_description=read('README.rst'),
    author='Steven Loria',
    author_email='sloria1@gmail.com',
    url='https://github.com/sloria/tinynetrc',
    install_requires=[],
    license='MIT',
    zip_safe=False,
    keywords='netrc posix',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    py_modules=['tinynetrc'],
)
